@@ -14,16 +14,19 @@
 
 ;;; Commentary:
 ;;
-;; el-get-core provides basic el-get API, intended for developpers of el-get
+;; el-get-core provides basic el-get API, intended for developers of el-get
 ;; and its methods.  See the methods directory for implementation of them.
 ;;
 
 (require 'dired)
-(require 'cl)            ; needed for `remove-duplicates'
+(require 'cl-lib)
 (require 'simple)        ; needed for `apply-partially'
 (require 'bytecomp)
 (require 'autoload)
 
+(declare-function el-get-package-def "el-get-recipes" (package))
+(declare-function el-get-installation-failed "el-get" (package signal-data))
+
 (defun el-get-print-to-string (object &optional pretty)
   "Return string representation of lisp object.
 
@@ -38,13 +41,24 @@ the original object."
 (defun el-get-verbose-message (format &rest arguments)
   (when el-get-verbose (apply 'message format arguments)))
 
+(defmacro el-get-with-errors-as-warnings (prefix &rest body)
+  (declare (indent 1) (debug t))
+  (let ((error-var (make-symbol "err")))
+    `(condition-case ,error-var
+         (progn ,@body)
+       ((debug error)
+        (display-warning 'el-get
+                         (concat ,prefix (error-message-string ,error-var))
+                         :error)
+        nil))))
+
 (defsubst el-get-plist-keys (plist)
   "Return a list of all keys in PLIST.
 
 Duplicates are removed."
-  (remove-duplicates
-   (loop for (k _) on plist by #'cddr
-         collect k)
+  (cl-remove-duplicates
+   (cl-loop for (k _) on plist by #'cddr
+            collect k)
    :test #'eq))
 
 (defsubst el-get-keyword-name (keyword)
@@ -75,32 +89,32 @@ call for doing the named package action in the given method.")
   "Returns t if NAME is a known el-get install method backend, nil otherwise."
   (and (el-get-method name :install) t))
 
-(defun* el-get-register-method (name &key install update remove
-                                     install-hook update-hook remove-hook
-                                     compute-checksum guess-website)
+(cl-defun el-get-register-method (name &key install update remove
+                                       install-hook update-hook remove-hook
+                                       compute-checksum guess-website)
   "Register the method for backend NAME, with given functions"
   (let (method-def)
-    (loop for required-arg in '(install update remove)
-          unless (symbol-value required-arg)
-          do (error "Missing required argument: :%s" required-arg)
-          do (setq method-def
-                   (plist-put method-def
-                              (intern (format ":%s" required-arg))
-                              (symbol-value required-arg))))
-    (loop for optional-arg in '(install-hook update-hook remove-hook
-                                compute-checksum guess-website)
-        if (symbol-value optional-arg)
-        do (setq method-def
-                 (plist-put method-def
-                            (intern (format ":%s" optional-arg))
-                            (symbol-value optional-arg))))
+    (cl-loop for required-arg in '(install update remove)
+             unless (symbol-value required-arg)
+             do (error "Missing required argument: :%s" required-arg)
+             do (setq method-def
+                      (plist-put method-def
+                                 (intern (format ":%s" required-arg))
+                                 (symbol-value required-arg))))
+    (cl-loop for optional-arg in '(install-hook update-hook remove-hook
+                                                compute-checksum guess-website)
+             if (symbol-value optional-arg)
+             do (setq method-def
+                      (plist-put method-def
+                                 (intern (format ":%s" optional-arg))
+                                 (symbol-value optional-arg))))
     (setq el-get-methods (plist-put el-get-methods name method-def))))
 
 (put 'el-get-register-method 'lisp-indent-function
      (get 'prog1 'lisp-indent-function))
 
-(defun* el-get-register-derived-method (name derived-from-name
-                                             &rest keys &key &allow-other-keys)
+(cl-defun el-get-register-derived-method (name derived-from-name
+                                               &rest keys &key &allow-other-keys)
   "Register the method for backend NAME.
 
 Defaults for all optional arguments are taken from
@@ -126,23 +140,25 @@ already-defined method DERIVED-FROM-NAME."
 ;; interface to write such structures as raw elisp.
 ;;
 ;;;  "Fuzzy" data structure conversion utilities
-(defun el-get-as-string (symbol-or-string)
-  "If STRING-OR-SYMBOL is already a string, return it.  Otherwise
-convert it to a string and return that."
-  (if (stringp symbol-or-string) symbol-or-string
-    (symbol-name symbol-or-string)))
+(defun el-get-as-string (obj)
+  "Return OBJ as a string."
+  (cond
+   ((stringp obj) obj)
+   ((symbolp obj) (symbol-name obj))
+   ((numberp obj) (number-to-string obj))
+   (t (error "Can't convert %S to string." obj))))
 
 (defun el-get-as-symbol (string-or-symbol)
   "If STRING-OR-SYMBOL is already a symbol, return it.  Otherwise
 convert it to a symbol and return that."
   (if (symbolp string-or-symbol) string-or-symbol
-      (intern string-or-symbol)))
+    (intern string-or-symbol)))
 
 (defun el-get-as-list (element-or-list)
   "If ELEMENT-OR-LIST is already a list, return it.  Otherwise
 returning a list that contains it (and only it)."
   (if (listp element-or-list) element-or-list
-      (list element-or-list)))
+    (list element-or-list)))
 
 (defun el-get-list-of-strings-p (obj)
   (or (null obj)
@@ -179,7 +195,45 @@ entry."
           ((file-exists-p pdir)
            (delete-file pdir)))
     (when post-remove-fun
-     (funcall post-remove-fun package))))
+      (funcall post-remove-fun package))))
+
+(defconst el-get-no-shell-quote "\\`[-,./_[:alnum:]]+\\'"
+  "Regular expression matching arguments that don't shell quoting.")
+
+(defun el-get-shell-quote-program (program-name)
+  "Like `shell-quote-argument' but needs special treatment on Windows."
+  (or (when (string-match-p el-get-no-shell-quote program-name) program-name)
+      (when (fboundp 'w32-short-file-name)
+        ;; If program is really a bat file, putting double quotes around
+        ;; it will lead to problems if subsequent arguments are also
+        ;; quoted. Use the short 8.3 name instead of quoting. See
+        ;; http://debbugs.gnu.org/cgi/bugreport.cgi?bug=18745 for
+        ;; details.
+        (let (exe (executable-find program-name))
+          (when exe (w32-short-file-name exe))))
+      (shell-quote-argument program-name)))
+
+(defun el-get-maybe-shell-quote-argument (arg)
+  "`shell-quote-argument', if necessary."
+  (if (string-match-p el-get-no-shell-quote arg) arg
+    (shell-quote-argument arg)))
+
+(defun el-get-read-from-file (filename)
+  "Read given FILENAME and return its content (a valid sexp is expected)."
+  (condition-case err
+      (with-temp-buffer
+        (insert-file-contents filename)
+        (read (current-buffer)))
+    ((debug error)
+     (error "Error reading file %s: %S" filename err))))
+
+(defun el-get-package-is-installed (package)
+  "Return true if PACKAGE is installed"
+  (and (file-directory-p (el-get-package-directory package))
+       (string= "installed"
+                (el-get-read-package-status package))))
+
+(defalias 'el-get-package-installed-p #'el-get-package-is-installed)
 
 
 ;;
@@ -187,11 +241,11 @@ entry."
 ;;
 (defun el-get-duplicates (list)
   "Return duplicates found in list."
-  (loop with dups and once
-	for elt in list
-	if (member elt once) collect elt into dups
-	else collect elt into once
-	finally return dups))
+  (cl-loop with dups and once
+           for elt in list
+           if (member elt once) collect elt into dups
+           else collect elt into once
+           finally return dups))
 
 (defun el-get-flatten (arg)
   "Return a version of ARG as a one-level list
@@ -202,11 +256,15 @@ entry."
       (apply 'append (mapcar 'el-get-flatten arg))
     (list arg)))
 
+(defun el-get-unquote (arg)
+  "Remote quote from ARG, if there is one."
+  (if (eq (car arg) 'quote) (nth 1 arg) arg))
+
 (defun el-get-load-path (package)
   "Return the list of absolute directory names to be added to
 `load-path' by the named PACKAGE."
   (let* ((source   (el-get-package-def package))
-	 (el-path  (if (plist-member source :load-path)
+         (el-path  (if (plist-member source :load-path)
                        (el-get-flatten (plist-get source :load-path))
                      '(".")))
          (pkg-dir (el-get-package-directory package)))
@@ -218,8 +276,8 @@ given method."
   (let* ((method  (if (keywordp method-name) method-name
                     (intern (concat ":" (format "%s" method-name)))))
          (actions (plist-get el-get-methods method)))
-    (assert actions nil
-            "Unknown recipe type: %s" method)
+    (cl-assert actions nil
+               "Unknown recipe type: %s" method)
     (plist-get actions action)))
 
 (defun el-get-check-init ()
@@ -231,16 +289,16 @@ given method."
   "Return the absolute directory name of the named PACKAGE."
   (file-name-as-directory
    (expand-file-name (el-get-as-string package)
-		     (expand-file-name el-get-dir))))
+                     (expand-file-name el-get-dir))))
 
 (defun el-get-add-path-to-list (package list path)
   "(add-to-list LIST PATH) checking for path existence within
 given package directory."
   (let* ((pdir     (el-get-package-directory package))
-	 (fullpath (expand-file-name (or path ".") pdir)))
+         (fullpath (expand-file-name (or path ".") pdir)))
     (unless (file-directory-p fullpath)
       (error "el-get could not find directory `%s' for package %s, at %s"
-	     path package fullpath))
+             path package fullpath))
     (add-to-list list fullpath)))
 
 (defun el-get-package-exists-p (package)
@@ -249,31 +307,41 @@ directory or a symlink in el-get-dir."
   (let ((pdir (el-get-package-directory package)))
     ;; seems overkill as file-directory-p will always be true
     (or (file-directory-p pdir)
-	(file-symlink-p   pdir))))
+        (file-symlink-p   pdir))))
+
+(defun el-get-url-host (url)
+  "Extract host from given URL.
+
+Earlier we used the built-in library `url-parse' to extract host. This broke
+installation of CEDET since it requires that the built-in versions of certain
+packages (one of them is `eieio') are not loaded before loading it. However
+`url-parse' depends on `auth-source' which in turn depends on `eieio' leading to
+loading of `eieio' before initializing CEDET causing CEDET's initialization to
+fail."
+  (string-match "://\\([^/:]+\\)" url)
+  (match-string-no-properties 1 url))
 
 
 ;;
 ;; el-get-reload API functions
 ;;
-(defun el-get-package-files (package)
-  "Return a list of files loaded from PACKAGE's directory."
-  (loop with pdir = (file-truename (el-get-package-directory package))
-        with regexp = (format "^%s" (regexp-quote (file-name-as-directory (expand-file-name pdir))))
-        for (f . nil) in load-history
-        when (and (stringp f) (string-match-p regexp (file-truename f)))
-        collect (if (string-match-p "\\.elc?$" f)
-                    (file-name-sans-extension f)
-                  f)))
+(defun el-get-package-files (pdir)
+  "Return a list of files loaded from directory PDIR."
+  (cl-loop with regexp = (format "^%s" (regexp-quote (file-name-as-directory (file-truename pdir))))
+           for (f . nil) in load-history
+           when (and (stringp f) (string-match-p regexp (file-truename f)))
+           collect (if (string-match-p "\\.elc?$" f)
+                       (file-name-sans-extension f)
+                     f)))
 
-(defun el-get-package-features (package)
-  "Return a list of features provided by files in PACKAGE."
-  (loop with pdir = (file-truename (el-get-package-directory package))
-        with regexp = (format "^%s" (regexp-quote (file-name-as-directory (expand-file-name pdir))))
-        for (f . l) in load-history
-        when (and (stringp f) (string-match-p regexp (file-truename f)))
-        nconc (loop for i in l
-                    when (and (consp i) (eq (car i) 'provide))
-                    collect (cdr i))))
+(defun el-get-package-features (pdir)
+  "Return a list of features provided by files in PDIR."
+  (cl-loop with regexp = (format "^%s" (regexp-quote (file-name-as-directory (expand-file-name pdir))))
+           for (f . l) in load-history
+           when (and (stringp f) (string-match-p regexp (file-truename f)))
+           nconc (cl-loop for i in l
+                          when (and (consp i) (eq (car i) 'provide))
+                          collect (cdr i))))
 
 
 ;;
@@ -378,73 +446,79 @@ makes it easier to conditionally splice a command into the list.
     (setq commands (cdr commands)))
   (condition-case err
       (if commands
-        (let* ((c       (car commands))
-               (next    (cdr commands))
-               (cdir    (plist-get c :default-directory))
-               (cname   (plist-get c :command-name))
-               (cbuf    (plist-get c :buffer-name))
-               (killed  (when (get-buffer cbuf) (kill-buffer cbuf)))
-               (filter  (plist-get c :process-filter))
-               (program (plist-get c :program))
-               (shell   (plist-get c :shell))
-               (args    (if shell
-			    (mapcar #'shell-quote-argument (plist-get c :args))
-			  (plist-get c :args)))
-               (sync    (el-get-plist-get-with-default c :sync
-                          el-get-default-process-sync))
-	       (stdin   (plist-get c :stdin))
-               (default-directory (if cdir
-                                      (file-name-as-directory
-                                       (expand-file-name cdir))
-                                    default-directory)))
-          (if sync
-              (progn
-                (el-get-verbose-message "Running commands synchronously: %S" commands)
-                (let* ((startf (if shell #'call-process-shell-command #'call-process))
-                       (infile (when stdin (make-temp-file "el-get")))
-                       (dummy  (when infile
-                                 (with-temp-file infile
-                                   (insert (el-get-print-to-string stdin)))))
-                       (dummy  (message "el-get is waiting for %S to complete" cname))
-                       (status (apply startf program infile cbuf t args))
-                       (message (plist-get c :message))
-                       (errorm  (plist-get c :error)))
-                  (when el-get-verbose
-                    (message "%S" (with-current-buffer cbuf (buffer-string))))
-                  (if (eq 0 status)
-                      (message "el-get: %s" message)
-                    (set-window-buffer (selected-window) cbuf)
-                    (error "el-get: %s %s" cname errorm))
-                  (when infile (delete-file infile))
-                  (when cbuf (kill-buffer cbuf))
-                  (if next
-                      ;; Prevent stack overflow on very long command
-                      ;; lists. This allows
-                      ;; `el-get-start-process-list' (but not other
-                      ;; functions) to recurse indefinitely.
-                      (let ((max-specpdl-size (+ 100 max-specpdl-size)))
-                        (el-get-start-process-list package next final-func))
-                    (when (functionp final-func)
-                      (funcall final-func package)))))
-            ;; async case
-            (el-get-verbose-message "Running commands asynchronously: %S" commands)
-            (let* ((startf (if shell #'start-process-shell-command #'start-process))
-                   (process-connection-type nil) ; pipe, don't pretend we're a pty
-                   (proc (apply startf cname cbuf program args)))
-              ;; add the properties to the process, then set the sentinel
-              (mapc (lambda (x) (process-put proc x (plist-get c x))) c)
-              (process-put proc :el-get-sources el-get-sources)
-              (process-put proc :el-get-package package)
-              (process-put proc :el-get-final-func final-func)
-              (process-put proc :el-get-start-process-list next)
-	      (when stdin
-		(process-send-string proc (el-get-print-to-string stdin))
-		(process-send-eof proc))
-              (set-process-sentinel proc 'el-get-start-process-list-sentinel)
-              (when filter (set-process-filter proc filter)))))
-	;; no commands, still run the final-func
-	(when (functionp final-func)
-	  (funcall final-func package)))
+          (let* ((c       (car commands))
+                 (next    (cdr commands))
+                 (cdir    (plist-get c :default-directory))
+                 (cname   (plist-get c :command-name))
+                 (cbuf    (plist-get c :buffer-name))
+                 (killed  (when (get-buffer cbuf) (kill-buffer cbuf)))
+                 (filter  (plist-get c :process-filter))
+                 (shell   (plist-get c :shell))
+                 (program (if shell
+                              (el-get-shell-quote-program (plist-get c :program))
+                            (plist-get c :program)))
+                 (args    (if shell
+                              (mapcar #'el-get-maybe-shell-quote-argument (plist-get c :args))
+                            (plist-get c :args)))
+                 (sync    (el-get-plist-get-with-default c :sync
+                            el-get-default-process-sync))
+                 (stdin   (plist-get c :stdin))
+                 (default-directory (if cdir
+                                        (file-name-as-directory
+                                         (expand-file-name cdir))
+                                      default-directory)))
+            (unless program (error "el-get: :program argument cannot be nil"))
+            (if sync
+                (progn
+                  (el-get-verbose-message "Running commands synchronously: %S" commands)
+                  (let* ((startf (if shell #'call-process-shell-command #'call-process))
+                         (infile (when stdin (make-temp-file "el-get")))
+                         (dummy  (when infile
+                                   (with-temp-file infile
+                                     (insert (el-get-print-to-string stdin)))))
+                         (dummy  (message "el-get is waiting for %S to complete" cname))
+                         (status (apply startf program infile cbuf t args))
+                         (message (plist-get c :message))
+                         (errorm  (plist-get c :error)))
+                    (when el-get-verbose
+                      (message "%S" (with-current-buffer cbuf (buffer-string))))
+                    (if (eq 0 status)
+                        (message "el-get: %s" message)
+                      (set-window-buffer (selected-window) cbuf)
+                      (error "el-get: %s %s" cname errorm))
+                    (when infile (delete-file infile))
+                    (when cbuf (kill-buffer cbuf))
+                    (if next
+                        ;; Prevent stack overflow on very long command
+                        ;; lists. This allows
+                        ;; `el-get-start-process-list' (but not other
+                        ;; functions) to recurse indefinitely.
+                        (let ((max-specpdl-size (+ 100 max-specpdl-size)))
+                          (el-get-start-process-list package next final-func))
+                      (when (functionp final-func)
+                        (funcall final-func package)))))
+              ;; async case
+              (el-get-verbose-message "Running commands asynchronously: %S" commands)
+              (let* ((process-connection-type nil) ; pipe, don't pretend we're a pty
+                     (proc (if shell
+                               (start-process-shell-command cname
+                                                            cbuf
+                                                            (mapconcat #'identity (cons program args) " "))
+                             (apply #'start-process cname cbuf program args))))
+                ;; add the properties to the process, then set the sentinel
+                (mapc (lambda (x) (process-put proc x (plist-get c x))) c)
+                (process-put proc :el-get-sources el-get-sources)
+                (process-put proc :el-get-package package)
+                (process-put proc :el-get-final-func final-func)
+                (process-put proc :el-get-start-process-list next)
+                (when stdin
+                  (process-send-string proc (el-get-print-to-string stdin))
+                  (process-send-eof proc))
+                (set-process-sentinel proc 'el-get-start-process-list-sentinel)
+                (when filter (set-process-filter proc filter)))))
+        ;; no commands, still run the final-func
+        (when (functionp final-func)
+          (funcall final-func package)))
     ((debug error)
      (el-get-installation-failed package err))))
 
@@ -466,7 +540,7 @@ Baring variable named \"el-get-NAME\", it will call
 `executable-find' on NAME and use the output of that, or error
 out if it's nil."
   (let ((fname (intern (format "el-get-%s-executable" name)))
-	(vname (intern (format "el-get-%s" name))))
+        (vname (intern (format "el-get-%s" name))))
     (cond
      ((fboundp fname)
       (funcall fname))
@@ -475,21 +549,21 @@ out if it's nil."
      ;; (bound-and-true-p vname) won't cut it
      ((ignore-errors (symbol-value vname))
       (let ((command (symbol-value vname)))
-	(unless (and (file-exists-p command)
-		     (file-executable-p command))
-	  (error
-	   (concat "The variable `%s' points to \"%s\", "
-		   "which is not an executable file name on your system.")
-	   name command))
-	command))
+        (unless (and (file-exists-p command)
+                     (file-executable-p command))
+          (error
+           (concat "The variable `%s' points to \"%s\", "
+                   "which is not an executable file name on your system.")
+           name command))
+        command))
 
      (t
       (let ((command (executable-find name)))
-	(unless command
-	  (error
-	   "The command named '%s' can not be found with `executable-find'"
-	   name))
-	command)))))
+        (unless command
+          (error
+           "The command named '%s' can not be found with `executable-find'"
+           name))
+        command)))))
 
 (defun el-get-plist-get-with-default (plist prop def)
   "Same as (plist-get PLIST PROP), but falls back to DEF.
